<?php
class Api {

	public	$has_error		= false,
					$socket 			=	null,
					$socket_error	= null;

	private	$_host = '127.0.0.1',
					$_port = '55542';

	/* [CONSTRUCTOR] */
	function __construct() {
		// check sbc-api running
		if ($this->_isBeronodeApiRunning()) {
			return;
		}
		// not running: setting up the error
		if (is_null($this->socket_error)) {
			$this->has_error = true;
			$this->socket_error = [
				'code'=> 500,
				'file'=> '',
				'line'=> '',
				'msg'	=> 'beronode not running (unknown error)',
			];
		}
	}

	/* [PUBLIC METHODS] */
	public function exec(string $cmd) {
		return $this->sendRequest('post', "/ExecuteCommand", ['cmd' => $cmd]);
	}

	public function hasError() {
		return $this->has_error;
	}

	public function getError($what = null) {
		return(is_null($what) ? $this->socket_error : $this->socket_error[$what]);
	}

	public function sendRequest($method, $uri, $data = null, $options = []) {
		if ($this->has_error) {
			return false;
		}
		try {
			// TODO: write a generic function in the Helper class
			$api_url = "http://{$this->_host}:{$this->_port}$uri";
	
			$ch = curl_init();
			if ($ch === false) {
				throw new Exception('Failed to initialize cURL');
			}

			curl_setopt($ch, CURLOPT_URL, $api_url);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

			switch ($method) {
			case 'delete':
				curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "DELETE");
				break;
			case 'get':
				break;
			case 'post':
				curl_setopt($ch, CURLOPT_POST, 1);
				break;
			case 'put':
				curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PUT");
				break;
			default:
				throw new Exception('Unknown cURL method');
			}

			if (!is_null($data)) {
				curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
				curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
			}
			curl_setopt($ch, CURLOPT_TIMEOUT, isset($options['timeout']) ? (int) $options['timeout'] : 2);

			$response = curl_exec($ch);
			if ($response === false) {
				throw new Exception(curl_error($ch), curl_errno($ch));
			}
			curl_close($ch);

			if ($response == false) {
				return true;
			}
			return(isset($options['withoutJsonDecode']) && $options['withoutJsonDecode'] ? $response : json_decode($response, true));
		}
		catch (Exception $e) {
			$this->has_error = true;
			$this->socket_error = [
				'code'=> $e->getCode(),
				'file'=> $e->getFile(),
				'line'=> $e->getLine(),
				'msg'	=> $e->getMessage(),
			];
			return false;
		}
	}

	/* [PRIVATE METHODS] */
	private function _isBeronodeApiRunning() {
		try {
			$this->socket = @fsockopen($this->_host, $this->_port, $errno, $errstr, 1);
		}
		catch (Exception $e) {
			$this->has_error = true;
			$this->socket_error = [
				'code'=> $e->getCode(),
				'file'=> $e->getFile(),
				'line'=> $e->getLine(),
				'msg'	=> $e->getMessage(),
			];
			return false;
		}
		return $this->socket;
	}
}
?>
